@@ -32,6 +32,14 @@
 #include <string.h>
 #include <math.h>
 
+#define CXML2C(s) ((char const *)(s))
+
+static inline gboolean
+attr_eq (const xmlChar *a, const char *s)
+{
+	return !strcmp (CXML2C (a), s);
+}
+
 /**
  * GOImageType:
  * @GO_IMAGE_STRETCHED: stretch the image so that it fills the whole area.
@@ -1265,15 +1273,22 @@ go_style_apply_theme (GOStyle *dst, GOStyle const *src, GOStyleFlag fields)
 	if ((fields & GO_STYLE_TEXT_LAYOUT) && dst->text_layout.auto_angle)
 		dst->text_layout.angle = src->text_layout.angle;
 
+	if (fields & GO_STYLE_FONT) {
+		if (dst->font.auto_color)
+			dst->font.color = src->font.color;
+
 #if 0
-	/* Fonts are not themed until we have some sort of auto mechanism
-	 * stronger than 'auto_size' */
-	if (src->font.font != NULL)
-		go_font_ref (src->font.font);
-	if (dst->font.font != NULL)
-		go_font_unref (dst->font.font);
-	dst->font = src->font;
+		/*
+		 * Fonts are not themed until we have some sort of auto
+		 * mechanism stronger than 'auto_size'
+		 */
+		if (src->font.font != NULL)
+			go_font_ref (src->font.font);
+		if (dst->font.font != NULL)
+			go_font_unref (dst->font.font);
+		dst->font = src->font;
 #endif
+	}
 }
 
 static void
@@ -1345,9 +1360,9 @@ static const struct {
 };
 
 static gboolean
-bool_sax_prop (char const *name, char const *id, char const *val, gboolean *res)
+bool_sax_prop (char const *name, xmlChar const *id, xmlChar const *val, gboolean *res)
 {
-	if (0 == strcmp (name, id)) {
+	if (attr_eq (id, name)) {
 		*res = g_ascii_tolower (*val) == 't' ||
 			g_ascii_tolower (*val) == 'y' ||
 			strtol (val, NULL, 0);
@@ -1408,7 +1423,9 @@ go_style_line_sax_save (GsfXMLOut *output, char const *name,
 		gsf_xml_out_add_cstr_unchecked (output, "dash",
 						go_line_dash_as_str (line->dash_type));
 
-	gsf_xml_out_add_float (output, "width", line->width, 6);
+	gsf_xml_out_add_bool (output, "auto-width", line->auto_width);
+	if (!line->auto_width)
+		gsf_xml_out_add_float (output, "width", line->width, 6);
 
 	gsf_xml_out_add_bool (output, "auto-color", line->auto_color);
 	if (!line->auto_color)
@@ -1514,12 +1531,21 @@ static void
 go_style_font_sax_save (GsfXMLOut *output, GOStyle const *style)
 {
 	char *str;
+
 	gsf_xml_out_start_element (output, "font");
-	go_xml_out_add_color (output, "color", style->font.color);
+
+	gsf_xml_out_add_bool (output, "auto-color", style->font.auto_color);
+	if (!style->font.auto_color)
+		go_xml_out_add_color (output, "color", style->font.color);
+
+	gsf_xml_out_add_bool (output, "auto-font", style->font.auto_font);
+	/* Unconditionally save font; theme support for fonts is incomplete */
 	str = go_font_as_str (style->font.font);
 	gsf_xml_out_add_cstr (output, "font", str);
 	g_free (str);
+
 	gsf_xml_out_add_bool (output, "auto-scale", style->font.auto_scale);
+
 	gsf_xml_out_end_element (output);
 }
 
@@ -1537,13 +1563,15 @@ go_style_sax_load_line (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	GOStyle *style = GO_STYLE (xin->user_state);
 	GOStyleLine *line = &style->line;
+	gboolean seen_auto_width = FALSE, seen_width = FALSE;
 
-	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "dash"))
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (attr_eq (attrs[0], "dash"))
 			line->dash_type = go_line_dash_from_str (attrs[1]);
 		else if (bool_sax_prop ("auto-dash", attrs[0], attrs[1], &line->auto_dash))
 			;
-		else if (0 == strcmp (attrs[0], "width")) {
+		else if (attr_eq (attrs[0], "width")) {
+			seen_width = TRUE;
 			line->width = g_strtod (attrs[1], NULL);
 			/* For compatibility with older graphs, when dash_type
 			 * didn't exist */
@@ -1551,10 +1579,21 @@ go_style_sax_load_line (GsfXMLIn *xin, xmlChar const **attrs)
 				line->width = 0.;
 				line->dash_type = GO_LINE_NONE;
 			}
-		} else if (0 == strcmp (attrs[0], "color"))
+		} else if (bool_sax_prop ("auto-width", attrs[0], attrs[1], &line->auto_width)) {
+			seen_auto_width = TRUE;
+		} else if (attr_eq (attrs[0], "color"))
 			go_color_from_str (attrs[1], &line->color);
 		else if (bool_sax_prop ("auto-color", attrs[0], attrs[1], &line->auto_color))
 			;
+	}
+
+	if (seen_width && !seen_auto_width) {
+		/*
+		 * Pre-0.10.16 lacked the auto-width attribute.  Let's just
+		 * assume the width was explicitly set iff it is non-zero.
+		 */
+		line->auto_width = (line->width == 0);
+	}
 }
 
 static void
@@ -1563,11 +1602,11 @@ go_style_sax_load_fill_pattern (GsfXMLIn *xin, xmlChar const **attrs)
 	GOStyle *style = GO_STYLE (xin->user_state);
 	g_return_if_fail (style->fill.type == GO_STYLE_FILL_PATTERN);
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "type"))
+		if (attr_eq (attrs[0], "type"))
 			style->fill.pattern.pattern = go_pattern_from_str (attrs[1]);
-		else if (0 == strcmp (attrs[0], "fore"))
+		else if (attr_eq (attrs[0], "fore"))
 			go_color_from_str (attrs[1], &style->fill.pattern.fore);
-		else if (0 == strcmp (attrs[0], "back"))
+		else if (attr_eq (attrs[0], "back"))
 			go_color_from_str (attrs[1], &style->fill.pattern.back);
 }
 
@@ -1577,13 +1616,13 @@ go_style_sax_load_fill_gradient (GsfXMLIn *xin, xmlChar const **attrs)
 	GOStyle *style = GO_STYLE (xin->user_state);
 	g_return_if_fail (style->fill.type == GO_STYLE_FILL_GRADIENT);
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "direction"))
+		if (attr_eq (attrs[0], "direction"))
 			style->fill.gradient.dir = go_gradient_dir_from_str (attrs[1]);
-		else if (0 == strcmp (attrs[0], "start-color"))
+		else if (attr_eq (attrs[0], "start-color"))
 			go_color_from_str (attrs[1], &style->fill.pattern.back);
-		else if (0 == strcmp (attrs[0], "end-color"))
+		else if (attr_eq (attrs[0], "end-color"))
 			go_color_from_str (attrs[1], &style->fill.pattern.fore);
-		else if (0 == strcmp (attrs[0], "brightness"))
+		else if (attr_eq (attrs[0], "brightness"))
 			go_style_set_fill_brightness (style, g_strtod (attrs[1], NULL));
 }
 
@@ -1597,11 +1636,11 @@ go_style_sax_load_fill_image (GsfXMLIn *xin, xmlChar const **attrs)
 	g_return_if_fail (style->fill.type == GO_STYLE_FILL_NONE);
 	g_return_if_fail (GO_IS_DOC (doc));
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "type"))
+		if (attr_eq (attrs[0], "type"))
 			style->fill.image.type = str_as_image_tiling (attrs[1]);
-		else if (0 == strcmp (attrs[0], "name"))
+		else if (attr_eq (attrs[0], "name"))
 			name = attrs[1];
-		else if (0 == strcmp (attrs[0], "type-name"))
+		else if (attr_eq (attrs[0], "type-name"))
 			type_name = attrs[1];
 	type = type_name? g_type_from_name (type_name): GO_TYPE_PIXBUF;
 	if (name && type)
@@ -1616,7 +1655,7 @@ go_style_sax_load_fill (GsfXMLIn *xin, xmlChar const **attrs)
 	GOStyle *style = GO_STYLE (xin->user_state);
 	style->fill.auto_type = FALSE;
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "type")) {
+		if (attr_eq (attrs[0], "type")) {
 			style->fill.type = str_as_fill_style (attrs[1]);
 			/* image fill can't be accepted until we have an image */
 			if (style->fill.type == GO_STYLE_FILL_IMAGE)
@@ -1638,19 +1677,19 @@ go_style_sax_load_marker (GsfXMLIn *xin, xmlChar const **attrs)
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
 		if (bool_sax_prop ("auto-shape", attrs[0], attrs[1], &style->marker.auto_shape))
 			;
-		else if (0 == strcmp (attrs[0], "shape"))
+		else if (attr_eq (attrs[0], "shape"))
 			go_marker_set_shape (marker, go_marker_shape_from_str (attrs[1]));
 		else if (bool_sax_prop ("auto-outline", attrs[0], attrs[1], &style->marker.auto_outline_color))
 			;
-		else if (0 == strcmp (attrs[0], "outline-color")) {
+		else if (attr_eq (attrs[0], "outline-color")) {
 			if (go_color_from_str (attrs[1], &c))
 				go_marker_set_outline_color (marker, c);
 		} else if (bool_sax_prop ("auto-fill", attrs[0], attrs[1], &style->marker.auto_fill_color))
 			;
-		else if (0 == strcmp (attrs[0], "fill-color")) {
+		else if (attr_eq (attrs[0], "fill-color")) {
 			if (go_color_from_str (attrs[1], &c))
 				go_marker_set_fill_color (marker, c);
-		} else if (0 == strcmp (attrs[0], "size"))
+		} else if (attr_eq (attrs[0], "size"))
 			go_marker_set_size (marker, g_strtod (attrs[1], NULL));
 
 	go_style_set_marker (style, marker);
@@ -1660,10 +1699,15 @@ static void
 go_style_sax_load_font (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	GOStyle *style = GO_STYLE (xin->user_state);
-	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "color"))
+	gboolean seen_auto_color = FALSE, seen_color = FALSE;
+
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (attr_eq (attrs[0], "color")) {
+			seen_color = TRUE;
 			go_color_from_str (attrs[1], &style->font.color);
-		else if (0 == strcmp (attrs[0], "font")) {
+		} else if (bool_sax_prop ("auto-color", attrs[0], attrs[1], &style->font.auto_color)) {
+			seen_auto_color = TRUE;
+		} else if (attr_eq (attrs[0], "font")) {
 			PangoFontDescription *desc = pango_font_description_from_string (attrs[1]);
 			if (desc != NULL) {
 				if (pango_font_description_get_family (desc) == NULL)
@@ -1672,14 +1716,27 @@ go_style_sax_load_font (GsfXMLIn *xin, xmlChar const **attrs)
 			}
 		} else if (bool_sax_prop ("auto-scale", attrs[0], attrs[1], &style->font.auto_scale))
 			;
+	}
+
+	if (seen_color && !seen_auto_color) {
+		/*
+		 * Pre-0.10.16 lacked the auto-color attribute.  Let's just
+		 * assume it was explicitly set iff it is not black.
+		 */
+		style->font.auto_color = (style->font.color == GO_COLOR_BLACK);
+	}
 }
+
 static void
 go_style_sax_load_text_layout (GsfXMLIn *xin, xmlChar const **attrs)
 {
 	GOStyle *style = GO_STYLE (xin->user_state);
-	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
-		if (0 == strcmp (attrs[0], "angle"))
+	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
+		if (attr_eq (attrs[0], "angle")) {
+			/* Note, that this sets auto-angle to FALSE.  */
 			go_style_set_text_angle (style, g_strtod (attrs[1], NULL));
+		}
+	}
 }
 
 static void
@@ -1923,7 +1980,7 @@ go_style_set_marker (GOStyle *style, GOMarker *marker)
  * Returns: (transfer none): the style #GOMarker.
  **/
 GOMarker const *
-go_style_get_marker (GOStyle *style)
+go_style_get_marker (GOStyle const *style)
 {
 	g_return_val_if_fail (GO_IS_STYLE (style), NULL);
 
